"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const case_sensitive_paths_webpack_plugin_1 = __importDefault(require("case-sensitive-paths-webpack-plugin"));
const clean_webpack_plugin_1 = require("clean-webpack-plugin");
const fork_ts_checker_webpack_plugin_1 = __importDefault(require("fork-ts-checker-webpack-plugin"));
const tsconfig_paths_webpack_plugin_1 = __importDefault(require("tsconfig-paths-webpack-plugin"));
const webpack_node_externals_1 = __importDefault(require("webpack-node-externals"));
const env_1 = require("../util/env");
const formatForkTsCheckerMessages_1 = require("./formatForkTsCheckerMessages");
const formatTsLoaderMessages_1 = require("./formatTsLoaderMessages");
const paths_1 = require("./paths");
const eslintFormatter_1 = __importDefault(require("./formatters/eslintFormatter"));
const eslintrc_1 = __importDefault(require("./eslintrc"));
exports.WebpackDevConfig = {
    mode: 'development',
    entry: paths_1.paths.appIndexJs,
    target: 'node',
    externals: [
        webpack_node_externals_1.default(env_1.RuntimeOptions.useMonorepo
            ? {
                modulesFromFile: true,
            }
            : undefined),
    ],
    devtool: 'inline-source-map',
    output: {
        path: paths_1.paths.appDevBundlePath,
        filename: 'bundle.js',
        libraryTarget: 'commonjs',
    },
    resolve: {
        extensions: ['.ts', '.tsx', '.js', 'jsx', '.json'],
        plugins: [
            new tsconfig_paths_webpack_plugin_1.default({
                configFile: paths_1.paths.appTsConfig,
                logLevel: 'ERROR',
            }),
        ],
    },
    module: {
        rules: [
            {
                test: /\.(j|t)sx?$/,
                enforce: 'pre',
                use: [
                    {
                        loader: require.resolve('eslint-loader'),
                        options: {
                            eslintPath: require.resolve('eslint'),
                            formatter: eslintFormatter_1.default,
                            useEslintrc: false,
                            emitWarning: true,
                            resolvePluginsRelativeTo: __dirname,
                            baseConfig: eslintrc_1.default
                        },
                    },
                ],
                include: paths_1.paths.appSrc,
            },
            {
                test: /\.jsx?$/,
                include: paths_1.paths.appSrc,
                exclude: /node_modules/,
                loader: require.resolve('babel-loader'),
                options: {
                    babelrc: false,
                    presets: [require.resolve('@babel/preset-env')],
                    compact: true,
                },
            },
            {
                test: /\.tsx?$/,
                include: paths_1.paths.appSrc,
                loader: require.resolve('ts-loader'),
                options: {
                    transpileOnly: true,
                    experimentalWatchApi: true,
                    errorFormatter: formatTsLoaderMessages_1.formatTsLoaderMessages,
                },
            },
        ],
    },
    plugins: [
        new fork_ts_checker_webpack_plugin_1.default({
            silent: true,
            async: false,
            watch: paths_1.paths.appSrc,
            tsconfig: paths_1.paths.appTsConfig,
            formatter: formatForkTsCheckerMessages_1.formatForkTsCheckerMessages,
            checkSyntacticErrors: true,
            reportFiles: ['**', '!**/__tests__/**', '!**/?(*.)(spec|test|t).*'],
        }),
        new case_sensitive_paths_webpack_plugin_1.default(),
        new clean_webpack_plugin_1.CleanWebpackPlugin(),
    ],
    optimization: {
        nodeEnv: false,
    },
    node: {
        __dirname: false,
        __filename: false,
    },
    performance: {
        hints: false,
    },
};
//# sourceMappingURL=webpack.config.dev.js.map