"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const typescript = __importStar(require("typescript"));
const webpack_1 = __importDefault(require("webpack"));
const webpack_merge_1 = __importDefault(require("webpack-merge"));
const formatWebpackMessages_1 = require("../lib/formatWebpackMessages");
const paths_1 = require("../lib/paths");
const webpack_config_prod_1 = require("../lib/webpack.config.prod");
const env_1 = require("../util/env");
const fileSizeReporter_1 = require("./util/fileSizeReporter");
const prodBundlePath = path.join(paths_1.paths.appBuild, paths_1.paths.prodBundle);
const sizeBeforeBuild = fileSizeReporter_1.getBundleSize(prodBundlePath);
const build = () => {
    let compiler;
    console.log(chalk_1.default.cyan('Creating an optimized production build...'));
    console.log(chalk_1.default.green('Using TypeScript v' + typescript.version));
    if (fs.existsSync(paths_1.paths.webpackOverride)) {
        console.log();
        console.log(chalk_1.default.yellow('[EXPERIMENTAL] Detected webpack.config.override.js file, merging configuration...'));
        // TODO: remove any once merge() updates to the latest webpack definitions
        const mergedConfig = webpack_merge_1.default(webpack_config_prod_1.WebpackProdConfig, require(paths_1.paths.webpackOverride));
        // TODO: remove `webpack.Configuration` once merge() updates to the latest webpack definitions
        compiler = webpack_1.default(mergedConfig);
    }
    else {
        compiler = webpack_1.default(webpack_config_prod_1.WebpackProdConfig);
    }
    return new Promise((resolve, reject) => {
        compiler.run((err, stats) => {
            if (err) {
                return reject(err);
            }
            const messages = formatWebpackMessages_1.formatWebpackMessages(stats.toJson());
            if (messages.errors.length) {
                if (messages.errors.length > 1 && !env_1.RuntimeOptions.noCollapse) {
                    messages.errors.length = 1;
                }
                return reject(new Error(messages.errors.join('\n\n')));
            }
            if (env_1.IS_CI && messages.warnings.length) {
                if (env_1.RuntimeOptions.bypassCiWarnings) {
                    console.log(chalk_1.default.yellow('\nBypassing warnings as CI errors due to --bypass-ci-warnings option.\n'));
                }
                else {
                    console.log(chalk_1.default.yellow('\nTreating warnings as errors because process.env.CI = true.\n' +
                        'Most CI servers set it automatically.\n'));
                    return reject(new Error(messages.warnings.join('\n\n')));
                }
            }
            return resolve({
                stats,
                warnings: messages.warnings,
            });
        });
    });
};
build()
    .then(({ warnings }) => {
    if (warnings.length) {
        console.log(chalk_1.default.yellow('Compiled with warnings.\n'));
        console.log(warnings.join('\n\n'));
        console.log('\nSearch for the ' +
            chalk_1.default.cyan('keywords') +
            ' to learn more about each warning.');
        console.log('To ignore, add ' +
            chalk_1.default.yellow('// eslint-disable-next-line') +
            ' to the line before.\n');
    }
}, err => {
    console.log(chalk_1.default.red('Failed to compile.\n'));
    console.log(err.message);
    console.log();
    process.exit(1);
})
    .then(() => {
    const sizeAfterBuild = fileSizeReporter_1.getBundleSize(prodBundlePath);
    console.log();
    console.log(chalk_1.default.greenBright('Successfully built bundle.prod.js!'));
    console.log('Bundle size: ' + fileSizeReporter_1.diffFileSize(sizeBeforeBuild, sizeAfterBuild));
    console.log();
})
    .catch(err => {
    if (err && err.message) {
        console.log(err.message);
    }
    process.exit(1);
});
//# sourceMappingURL=build.js.map